use opossum::{
    analyzers::{AnalyzerType, RayTraceConfig},
    degree,
    error::OpmResult,
    joule, millimeter,
    nodes::{
        collimated_line_ray_source, NodeGroup, NodeReference, RayPropagationVisualizer,
        SpotDiagram, ThinMirror,
    },
    optic_node::Alignable,
    OpmDocument,
};
use std::path::Path;

fn main() -> OpmResult<()> {
    let mut scenery = NodeGroup::default();
    let i_src = scenery.add_node(&collimated_line_ray_source(
        millimeter!(20.0),
        joule!(1.0),
        10,
    )?)?;
    let i_m1 =
        scenery.add_node(&ThinMirror::new("mirror 1").with_tilt(degree!(45.0, 0.0, 0.0))?)?;
    let i_m2 = scenery.add_node(&ThinMirror::new("mirror 2").with_tilt(degree!(2.0, 0.0, 0.0))?)?;
    let m1_ref = NodeReference::from_node(&scenery.node(i_m1)?);
    let i_m1_ref = scenery.add_node(&m1_ref)?;
    let i_sd3 = scenery.add_node(&RayPropagationVisualizer::default())?;
    let i_sd = scenery.add_node(&SpotDiagram::default())?;
    let sd_ref = NodeReference::from_node(&scenery.node(i_sd)?);
    let i_sd_ref = scenery.add_node(&sd_ref)?;
    scenery.connect_nodes(i_src, "out1", i_sd, "in1", millimeter!(40.0))?;
    scenery.connect_nodes(i_sd, "out1", i_m1, "input", millimeter!(40.0))?;
    scenery.connect_nodes(i_m1, "reflected", i_m2, "input", millimeter!(50.0))?;
    scenery.connect_nodes(i_m2, "reflected", i_m1_ref, "input", millimeter!(0.0))?;
    scenery.connect_nodes(i_m1_ref, "reflected", i_sd_ref, "in1", millimeter!(0.0))?;
    scenery.connect_nodes(i_sd_ref, "out1", i_sd3, "in1", millimeter!(20.0))?;

    let mut doc = OpmDocument::new(scenery);
    doc.add_analyzer(AnalyzerType::RayTrace(RayTraceConfig::default()));
    doc.save_to_file(Path::new("./opossum/playground/mirror_inverse.opm"))
}
